/*
 * lab0.c
 *
 */


#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>

#define SET_ADC_CH(ch) {	\
	ADMUX&=0b11111000;		\
	ADMUX|=((ch)&7);		\
	}

int input[5] = {PA0, PA1, PA2, PA3, PA4};
int lines[8] = {PA5, PA6, PA7, PC4, PC3, PC2, PC1, PC0};
int col_up[5] = {PD5, PD3, PD1, PB0, PB2};
int col_down[5] = {PD4, PD2, PD0, PB1, PB3};

int levels[7] = {40, 55, 65, 80, 95, 110, 130};

int i,j;


/* pregateste porturile de intrare/iesire */
void prepare() {
	
	// input
	for (i=0; i<5; i++){
		DDRA &= ~(1 << input[i]);
	}
	
	// ADC
	ADCSRA = 0;
	ADCSRB = 0;
	ADMUX = 0;
	ADMUX = (1<<REFS0);				// For Aref=AVcc;
	ADCSRA = (1<<ADEN) | (1<<ADPS0)| (1<<ADPS1) | (0<<ADPS2); 	//Prescalar div factor = 128
	
	// output
	DDRA |= ((1 << PA7) | (1 << PA6) | (1 << PA5));
	
	// output (leds)
	for (i=0; i<8; i++){
		DDRC |= (1 << lines[i]);
	}
	
	DDRB |= ((1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3));
	DDRD |= ((1 << PD0) | (1 << PD1) | (1 << PD2) | (1 << PD3) | (1 << PD4) | (1 << PD5));
}

uint16_t capt(){
	ADCSRA|=(1<<ADSC);
	while(!(ADCSRA & (1<<ADIF)));

	ADCSRA|=(1<<ADIF);

	return(ADC);
}

/* reseteaza o linie */
void reset_line(int i) {
	switch(i){
		case 0:
		case 1:
		case 2:
			PORTA |= (1 << lines[i]);
			break;
		default:
			PORTC |= (1 << lines[i]);
	}
}

/* reseteaza o coloana */
void reset_col(int i){
	switch(i){
		case 0:
		case 1:
		case 2:
			PORTD &= ~((1 << col_up[i]) | (1 << col_down[i]));
			break;
		case 3:
		case 4:
			PORTB &= ~((1 << col_up[i]) | (1 << col_down[i]));
			break;
	}
}

/* reseteaza toate led-urile */
void reset() {
	
	// lines
	for (i=0; i<8; i++){
		reset_line(i);
	}
	
	// cols
	for (i=0; i<5; i++){
		reset_col(i);
	}
	
}

/* aprinde o linie */
void set_line(int i){
	switch(i){
		case 0:
		case 1:
		case 2:
			PORTA &= ~(1 << lines[i]);
			break;
		default:
			PORTC &= ~(1 << lines[i]);
	}
}

/* aprinde o coloana */
void set_col(int i){
	switch(i){
		case 0:
		case 1:
		case 2:
			PORTD |= ((1 << col_up[i]) | (1 << col_down[i]));
			break;
		case 3:
		case 4:
			PORTB |= ((1 << col_up[i]) | (1 << col_down[i]));
			break;
	}
}

/* stabileste nivelul in functie de semnalul primit */
int get_level(int val) {
	int q;
	
	for (q=0; q<7; q++){
		if (val <= levels[q])
			return q;
	}
	
	return q;	// max level
}

/* aprinde o coloana pana la un anumit nivel (in functie de prag) */
void set_level(int col, int lev){

	int q;
	set_col(col);
	for (q=0; q<lev+1; q++){
		set_line(q);
	}
	
	
}

/* obtine semnalul asociat unei coloane */
int get(int col){
	int r = 0;
	// trebuie sa citesc semnalul primit pe pinul input[q];
	
	return r;
}

int lev[5] = {0,1,2,3,4};
int sens[5] = {0,0,0,0,0};

void test1() {
	int q, qi;
	while(1){
		for (qi=0; qi<500; qi++){
			for (q=0; q<5; q++){
				reset();
				set_level(q, lev[q]+1);
			}
		}
		for (q=0; q<5; q++){
			if (sens[q] == 0)
				lev[q]++;
			else
				lev[q]--;
			if (lev[q] % 5 == 0){
				sens[q] = 1 - sens[q];
			}
		}
	}
}

int main(void) {
	prepare();
	reset();
	int q, qi;
	int val=0, rez;
	int last_values[5] = {0,0,0,0,0};
	int values[5] = {0,0,0,0,0};
	
	//test1();
	
	while(1){
		for (q=0; q<5; q++){
			SET_ADC_CH(input[q]);
			
			if(capt() > 0) {
				val++;
			}
			else 
				val--;
			
			rez = val % 8;
			if (rez > last_values[q]){
				values[q]++;
			}else if (rez < last_values[q]){
				values[q]--;
			}
		}
		for (qi=0; qi<300; qi++){
			for (q=0; q<5; q++){
				reset();
				set_level(q, values[q]);
				last_values[q] = values[q];
			}
		}
	}
	
	return 0;
}

